/* Rohde & Schwarz - AMIQ Bit Error Rate Tests ==============================*/
/* Original Release: September 1999											 */
/* Rev 1.0, 09/1999, CVI 5.0.1					 							 */
/* Compatible with instr driver revision: "Rev 1.0, 9/1999, CVI 5.0.1"		 */
/* By: PP, ELCOM, a.s., CZ											 		 */
/*                                                                           */
/* Modification History: 													 */
/*																			 */
/*		None																 */
/*																			 */
/*===========================================================================*/

#include <ansi_c.h>
#include <cvirte.h>		/* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include "Bit Error Rate Tests.h"
#include "export.h"
#include "rsamiq.h"

/*= Definitions =============================================================*/
#define checkERR(fcal) if ((checkERR = fcal) < VI_SUCCESS) {					\
        if (checkERR >= 0xBFFF0000) {							    			\
            abortApplication ();    							    			\
        }														    			\
        else {													    			\
            rsamiq_error_message (VI_NULL, checkERR, errMsg);        			\
            if (checkERR == rsamiq_ERROR_INSTRUMENT_ERROR)		    			\
				rsamiq_error_query (session, &checkERR, errMsg);				\
            SetWaitCursor(0);									    			\
            MessagePopup ("Error", errMsg);						    			\
            return 0;											    			\
        }   													    			\
    } else

/*= Global Variable Definitions =============================================*/
extern ViSession session;
static int panelHandle;
static ViInt32 checkERR;
static ViChar errMsg[256];
static int type;
static int cPol;
static int dPol;
static short mask;
static unsigned int bCount;
static unsigned int eCount;
static short trigg;
static int dBits;
static int eBits;
static double eRate;
static short tStatus;
static short cLStatus;
static short dLStatus;
static short sStatus;

/*=================== Utility Function Declarations =========================*/
long abortApplication (void);

/*= Main Function ===========================================================*/
int main (int argc, char *argv[])
{
	if (InitCVIRTE (0, argv, 0) == 0)	/* Needed if linking in external compiler; harmless otherwise */
		return -1;	/* out of memory */
	if ((panelHandle = LoadPanel (0, "Bit Error Rate Tests.uir", PANEL)) < 0)
		return -1;
	ConnectionDialog(panelHandle);
	RunUserInterface ();
	return 0;
}

/* Quit callback */
int CVICALLBACK Quit (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			QuitUserInterface (0);
			checkERR(rsamiq_close (session));
			break;
	}
	return 0;
}

/* Scan all controls and write to AMIQ */
int CVICALLBACK update (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_VAL_CHANGED:
			GetCtrlVal (panelHandle, PANEL_TYPE, &type);
			GetCtrlVal (panelHandle, PANEL_CPOLAR, &cPol);			
			GetCtrlVal (panelHandle, PANEL_DPOLAR, &dPol);						
			GetCtrlVal (panelHandle, PANEL_MASK, &mask);						
			GetCtrlVal (panelHandle, PANEL_BCOUNT, &bCount);						
			GetCtrlVal (panelHandle, PANEL_ECOUNT, &eCount);						
			GetCtrlVal (panelHandle, PANEL_TRIGG, &trigg);						
			
			checkERR (rsamiq_EasyBERTConfig (session, type, dPol, cPol,
								   mask, bCount, eCount, trigg));

			break;
		}
	return 0;
}

/* Read and parse BERT result */
int CVICALLBACK readRes (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	char text[100];
	switch (event)
		{
		case EVENT_COMMIT:
			 checkERR (rsamiq_GetBERTResult (session, &dBits, &eBits, &eRate, &tStatus,
											   &cLStatus, &dLStatus, &sStatus));
		 	 ResetTextBox (panelHandle, PANEL_RESULT, "");
			 
			 sprintf(text,"No. of Data bits: %d",dBits);
			 InsertTextBoxLine (panelHandle, PANEL_RESULT, -1, text);

			 sprintf(text,"No. of Error bits: %d",eBits);
			 InsertTextBoxLine (panelHandle, PANEL_RESULT, -1, text);

			 sprintf(text,"Bit/Error rate: %lf",eRate);
			 InsertTextBoxLine (panelHandle, PANEL_RESULT, -1, text);

			 sprintf(text,"Termination status: %s",tStatus?"Terminated":"Not yet terminated");
			 InsertTextBoxLine (panelHandle, PANEL_RESULT, -1, text);

			 sprintf(text,"Clock line status: %s",cLStatus?"Active":"Not Active");
			 InsertTextBoxLine (panelHandle, PANEL_RESULT, -1, text);

			 sprintf(text,"Data line status: %s",dLStatus?"Active":"Not Active");
			 InsertTextBoxLine (panelHandle, PANEL_RESULT, -1, text);

			 sprintf(text,"Sync status: %s",dLStatus?"synchronized":"Not synchronized");
			 InsertTextBoxLine (panelHandle, PANEL_RESULT, -1, text);

			break;
		}
	return 0;
}

/*===========================================================================*/
/* Function: abortApplication
/* Purpose:  
/*===========================================================================*/
long abortApplication (void)
{
    ViChar errorMessage[256], errBuffer[256];
    ViInt16 response;

    SetWaitCursor (0);
    rsamiq_error_message(VI_NULL, checkERR, errorMessage);
    sprintf (errBuffer, "The following error caused the application to fail:\n\n%s\n\n"
    	"Attempting to Close Instrument Driver and Exit Application", errorMessage);
    MessagePopup("ERROR!, Application Failed", errBuffer);
	if ((checkERR = rsamiq_close (session)) < 0) {
        rsamiq_error_message(VI_NULL, checkERR, errorMessage);
        sprintf (errBuffer, "Application failed to close the Instrument Driver.\n\n"
            "%s\n\nExiting Application.", errorMessage);
        MessagePopup("ERROR!",errBuffer);
    }
    else 
        MessagePopup("Application Aborted","Instrument Driver Closed, Exiting Application.");
    QuitUserInterface (checkERR);
    exit (-1);
    
    return 0;
}

/* End of file "Bit Error Rate Tests.c" */
