/* Rohde & Schwarz - AMIQ Marker Settings ===================================*/
/* Original Release: September 1999											 */
/* Rev 1.0, 09/1999, CVI 5.0.1					 							 */
/* Compatible with instr driver revision: "Rev 1.0, 9/1999, CVI 5.0.1"		 */
/* By: PP, ELCOM, a.s., CZ											 		 */
/*                                                                           */
/* Modification History: 													 */
/*																			 */
/*		None																 */
/*																			 */
/*===========================================================================*/

#include "rsamiq.h"
#include <ansi_c.h>
#include <cvirte.h>		/* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include "Marker Settings.h"
#include "export.h"

/*= Definitions =============================================================*/
#define MAX_MARKER_LIST 1000
#define LAST_MARKER		3999999

#define checkERR(fcal) if ((checkERR = fcal) < VI_SUCCESS) {					\
        if (checkERR >= 0xBFFF0000) {							    			\
            abortApplication ();    							    			\
        }														    			\
        else {													    			\
            rsamiq_error_message (VI_NULL, checkERR, errMsg);        			\
            if (checkERR == rsamiq_ERROR_INSTRUMENT_ERROR)		    			\
				rsamiq_error_query (session, &checkERR, errMsg);				\
            SetWaitCursor(0);									    			\
            MessagePopup ("Error", errMsg);						    			\
            return 0;											    			\
        }   													    			\
    } else

/*= Global Variable Definitions =============================================*/

static int panelHandle;
static ViInt32 checkERR;
static ViChar errMsg[256];

static int idxCtrls[]={PANEL_SIDX_1,PANEL_SIDX_2,PANEL_SIDX_3,PANEL_SIDX_4,
					   PANEL_SIDX_5};
static int stateCtrls[]={PANEL_MSTATE_1,PANEL_MSTATE_2,PANEL_MSTATE_3,
						 PANEL_MSTATE_4,PANEL_MSTATE_5};
static int itemBarCtrls[]={PANEL_MBAR_1,PANEL_MBAR_2,PANEL_MBAR_3,PANEL_MBAR_4,
						   PANEL_MBAR_5};	

extern ViSession session;

static int marker = 0;
static int idxs[4][MAX_MARKER_LIST];
static int states[4][MAX_MARKER_LIST];
static int topIdx[4]={0,0,0,0};
static int items[4]={0,0,0,0};

/*=================== Utility Function Declarations =========================*/
long abortApplication (void);
int UpdateMakerList(int *idxs, int *states,int items);

/*= Main Function ===========================================================*/
int main (int argc, char *argv[])
{
	if (InitCVIRTE (0, argv, 0) == 0)	/* Needed if linking in external compiler; harmless otherwise */
		return -1;	/* out of memory */
	if ((panelHandle = LoadPanel (0, "Marker Settings.uir", PANEL)) < 0)
		return -1;
	ConnectionDialog(panelHandle);
	UpdateMakerList(NULL,NULL,0);
	RunUserInterface ();
	return 0;
}

/* Write marker list to the AMIQ */
int SetMarker(int *idxs, int *states,  int items){
   static int start[MAX_MARKER_LIST + 2];
   static int stop[MAX_MARKER_LIST + 2];
   static short state[MAX_MARKER_LIST + 2];
   static int i;
   int 	cnt = 0;

   		if(!items) return 0;
   		
   		/* Add Start condition "0 - idxs[0]:0" as needed */
   		if(idxs[0]){
   		  start[0]	= 0;
   		  stop[0] 	= idxs[0]-1;
   		  state[cnt++] 		= 0;
		}   		  

  		/* Regular loop */
  		for(i=0;i<(items-1);i++){
  		  start[cnt] = idxs[i];
  		  stop[cnt] = idxs[i+1]-1;
  		  state[cnt++] = states[i];
  		}
   		
		/* Add Final condition as needed */
		if(idxs[i]<LAST_MARKER) {
		  start[cnt] = idxs[i];
		  stop[cnt] = LAST_MARKER;
		  state[cnt++] = states[i];
		}

		/* Update device */
		checkERR (rsamiq_SetMarkers (session, marker+1, start, stop, state, cnt));
		
		return 0;
}

/* Update marker list */
int UpdateMakerList(int *idxs, int *states,int items)
{
	int i;
	char text[20];

		for(i=0;i<5;i++){
		   SetCtrlAttribute (panelHandle,idxCtrls[i], ATTR_DIMMED,i>=items);
		   SetCtrlAttribute (panelHandle,stateCtrls[i], ATTR_DIMMED,i>=items);		  
		   
		   if(i>=items)
		   		{
		     		SetCtrlVal (panelHandle,idxCtrls[i],"0");
			 		SetCtrlVal (panelHandle,stateCtrls[i],"0");		   
		 		}	
		 		else
		 		{
		     		sprintf(text,"%d",idxs[i]);
		     		SetCtrlVal (panelHandle,idxCtrls[i],text);
					sprintf(text,"%d",states[i]);		     		
			 		SetCtrlVal (panelHandle,stateCtrls[i],text);		   
		 		}
		   /* Hide selection bar */
		   checkERR (SetCtrlAttribute (panelHandle, itemBarCtrls[i], ATTR_VISIBLE, 0));
		}

	return 0;
}

/* Show/hide controls respectively */
int CVICALLBACK ActiveItem (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int i,j;
		switch (event) {
			case EVENT_LEFT_CLICK:
		  		for(i=0;i<5;i++) if(control==idxCtrls[i]) break;
		  		if(i==5) for(i=0;i<5;i++) if(control==stateCtrls[i]) break;
				for(j=0;j<5;j++)  	  
				  SetCtrlAttribute (panelHandle, itemBarCtrls[j], ATTR_VISIBLE,j==i?1:0);
				  SetCtrlVal (panelHandle,PANEL_SAMPLEIDX,idxs[marker][i+topIdx[marker]]);
				  SetCtrlVal (panelHandle,PANEL_STATE,states[marker][i+topIdx[marker]]);				  	
				break;
	}
	return 0;
}

/* Add new marker */
int CVICALLBACK AddMarker (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int i,j,idx,state;	
	switch (event) {
		case EVENT_COMMIT:
		   GetCtrlVal (panelHandle, PANEL_SAMPLEIDX, &idx);	
		   GetCtrlVal (panelHandle, PANEL_STATE, &state);			   
		   for(i=0;i<items[marker];i++){
		     if(idxs[marker][i]==idx){
			   /* Replace */
			   if(!ConfirmPopup ("Confirm Overwrite",
								  "Marker point with this sample index already exists. Overwrite?")) return 0;
		       
		       break;
		     }

			 /* Add between */
			 if(idxs[marker][i]>idx){
			   for(j=items[marker];j>i;j--){
			     idxs[marker][j]=idxs[marker][j-1];
			     states[marker][j]=states[marker][j-1];
		       }
  		       items[marker]++;
			   break;	
		     } 
		   }
		   /* Add on tail */
		   if(i==items[marker]) items[marker]++;	

		   /* Update marker list */							
		   idxs[marker][i]=idx;
		   states[marker][i]=state;
		   SetCtrlAttribute (panelHandle, PANEL_SLIDE, ATTR_MAX_VALUE, ((items[marker]-5)<0)?0:items[marker]-5);
		   topIdx[marker]=((i-4)<0)?0:i-4;
		   SetCtrlVal (panelHandle, PANEL_SLIDE,(((items[marker]-5)<0)?0:items[marker]-5)-topIdx[marker]);
		   UpdateMakerList(idxs[marker]+topIdx[marker],states[marker]+topIdx[marker],items[marker]-topIdx[marker]);			  			
		break;
	}
	return 0;
}

int CVICALLBACK CurrentMarker (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_VAL_CHANGED:
			  GetCtrlVal (panelHandle, PANEL_MARKER, &marker);
			  UpdateMakerList(idxs[marker]+topIdx[marker],states[marker]+topIdx[marker],items[marker]-topIdx[marker]);			  
			break;
	}
	return 0;
}

int CVICALLBACK DeleteMarker (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int i,j,idx;
	switch (event) {
		case EVENT_COMMIT:

			/* List is empty */
			if(!items[marker]) return 0;
		   
			/* Get idx to delete */
			GetCtrlVal (panelHandle, PANEL_SAMPLEIDX, &idx);	
		   
			/* Go throught the marker list */ 
			for(i=0;i<items[marker];i++){
		     
				/* Check index */
				if(idxs[marker][i]==idx){
				
					for(j=i;j<(items[marker]-1);j++){
						idxs[marker][j] = idxs[marker][j+1];
						states[marker][j] = states[marker][j+1];
					}
			
					items[marker]--;			  
			   
					break;
				}
		     }

		   /* Update marker list */							
		   SetCtrlAttribute (panelHandle, PANEL_SLIDE, ATTR_MAX_VALUE, ((items[marker]-5)<0)?0:items[marker]-5);
		   topIdx[marker]=((i-4)<0)?0:i-4;
		   SetCtrlVal (panelHandle, PANEL_SLIDE,(((items[marker]-5)<0)?0:items[marker]-5)-topIdx[marker]);
		   UpdateMakerList(idxs[marker]+topIdx[marker],states[marker]+topIdx[marker],items[marker]-topIdx[marker]);			  			
		break;
	}
	return 0;
}

/* Reset marker list callback */
int CVICALLBACK ResetList (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			topIdx[marker]=0;
			items[marker]=0;
		    UpdateMakerList(NULL,NULL,0);
		    SetCtrlVal (panelHandle,PANEL_SAMPLEIDX,0);
		    SetCtrlVal (panelHandle,PANEL_STATE,0);
			break;
	}
	return 0;
}

/* Ok call back */
int CVICALLBACK ok (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			  SetMarker(idxs[marker], states[marker], items[marker]);
			break;
	}
	return 0;
}

/* Cancel button callback */
int CVICALLBACK cancel (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			QuitUserInterface (0);
			break;
	}
	return 0;
}

/* Slider callback */
int CVICALLBACK Slide (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int max;
	switch (event) {
		case EVENT_VAL_CHANGED:
			  GetCtrlVal (panelHandle, PANEL_SLIDE, &topIdx[marker]);
			  GetCtrlAttribute (panelHandle, PANEL_SLIDE, ATTR_MAX_VALUE, &max);			 
			  topIdx[marker]=max-topIdx[marker];
			  UpdateMakerList(idxs[marker]+topIdx[marker],states[marker]+topIdx[marker],items[marker]-topIdx[marker]);			  
			break;
	}
	return 0;
}

/*===========================================================================*/
/* Function: abortApplication
/* Purpose:  
/*===========================================================================*/
long abortApplication (void)
{
    ViChar errorMessage[256], errBuffer[256];
    ViInt16 response;

    SetWaitCursor (0);
    rsamiq_error_message(VI_NULL, checkERR, errorMessage);
    sprintf (errBuffer, "The following error caused the application to fail:\n\n%s\n\n"
    	"Attempting to Close Instrument Driver and Exit Application", errorMessage);
    MessagePopup("ERROR!, Application Failed", errBuffer);
	if ((checkERR = rsamiq_close (session)) < 0) {
        rsamiq_error_message(VI_NULL, checkERR, errorMessage);
        sprintf (errBuffer, "Application failed to close the Instrument Driver.\n\n"
            "%s\n\nExiting Application.", errorMessage);
        MessagePopup("ERROR!",errBuffer);
    }
    else 
        MessagePopup("Application Aborted","Instrument Driver Closed, Exiting Application.");
    QuitUserInterface (checkERR);
    exit (-1);
	
	return 0;
}

/* End of "Marker Setting.c" */

