/* Rohde & Schwarz - AMIQ Output Settings ===================================*/
/* Original Release: September 1999											 */
/* Rev 1.0, 09/1999, CVI 5.0.1					 							 */
/* Compatible with instr driver revision: "Rev 1.0, 9/1999, CVI 5.0.1"		 */
/* By: PP, ELCOM, a.s., CZ											 		 */
/*                                                                           */
/* Modification History: 													 */
/*																			 */
/*		None																 */
/*																			 */
/*===========================================================================*/

#include <ansi_c.h>
#include <cvirte.h>		/* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include "Output Settings.h"
#include "export.h"
#include "rsamiq.h"	

/*= Definitions =============================================================*/
#define checkERR(fcal) if ((checkERR = fcal) < VI_SUCCESS) {					\
        if (checkERR >= 0xBFFF0000) {							    			\
            abortApplication ();    							    			\
        }														    			\
        else {													    			\
            rsamiq_error_message (VI_NULL, checkERR, errMsg);        			\
            if (checkERR == rsamiq_ERROR_INSTRUMENT_ERROR)		    			\
				rsamiq_error_query (session, &checkERR, errMsg);				\
            SetWaitCursor(0);									    			\
            MessagePopup ("Error", errMsg);						    			\
            return 0;											    			\
        }   													    			\
    } else

#define STATE_OFF  0
#define STATE_FIX  1
#define STATE_VAR  2
#define STATE_INV  3

#define TYPE_BAL   0
#define TYPE_UNBAL 1

/*= Global Variable Definitions =============================================*/
extern ViSession session;
static ViInt32 checkERR;
static ViChar errMsg[256];
static int panelHandle;
static int outType;
static double amplI;
static double amplQ;
static double biasI;
static double biasQ;
static int stateI;
static int stateQ;
static int imped;
static int filtI;
static int filtQ;

/*=================== Utility Function Declarations =========================*/
long abortApplication (void);
void UpdatePanel(void);

/*= Main Function ===========================================================*/
int main (int argc, char *argv[])
{
	if (InitCVIRTE (0, argv, 0) == 0)	/* Needed if linking in external compiler; harmless otherwise */
		return -1;	/* out of memory */
	if ((panelHandle = LoadPanel (0, "Output Settings.uir", PANEL)) < 0)
		return -1;
	UpdatePanel();
	ConnectionDialog(panelHandle);
	RunUserInterface ();
	return 0;
}

/* Set controls attributes as needed */
void UpdatePanel(void){
  GetCtrlVal (panelHandle, PANEL_OUTTYPE, &outType);
  SetCtrlAttribute (panelHandle, PANEL_BIAS_I, ATTR_DIMMED, !outType);
  SetCtrlAttribute (panelHandle, PANEL_BIAS_Q, ATTR_DIMMED, !outType);   
  SetCtrlAttribute (panelHandle, PANEL_AMPL_I, ATTR_MAX_VALUE,!outType?1.0:4.0);   
  SetCtrlAttribute (panelHandle, PANEL_AMPL_Q, ATTR_MAX_VALUE,!outType?1.0:4.0);
  
  GetCtrlVal (panelHandle, PANEL_STATE_I, &stateI);
  GetCtrlVal (panelHandle, PANEL_STATE_Q, &stateQ);
  if(stateI==STATE_VAR || stateI==STATE_INV){
    SetCtrlAttribute (panelHandle, PANEL_AMPL_I, ATTR_DIMMED,0);
  }else{
    SetCtrlAttribute (panelHandle, PANEL_AMPL_I, ATTR_DIMMED,1);
  }

  if(stateQ==STATE_VAR || stateQ==STATE_INV){
    SetCtrlAttribute (panelHandle, PANEL_AMPL_Q, ATTR_DIMMED,0);
  }else{
    SetCtrlAttribute (panelHandle, PANEL_AMPL_Q, ATTR_DIMMED,1);
  }
}


/* Set output type */
int CVICALLBACK OutType (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
		  UpdatePanel();
			break;
	}
	return 0;
}

/* Set state */
int CVICALLBACK State (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			 UpdatePanel();
			break;
	}
	return 0;
}

/* Quit user interface */
int CVICALLBACK Quit (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			QuitUserInterface (0);
			checkERR (rsamiq_close (session));
			break;
	}
	return 0;
}

/* Set output */
int CVICALLBACK set (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
		  GetCtrlVal (panelHandle, PANEL_AMPL_I, &amplI);
		  GetCtrlVal (panelHandle, PANEL_AMPL_Q, &amplQ);		  
		  GetCtrlVal (panelHandle, PANEL_BIAS_I, &biasI);
		  GetCtrlVal (panelHandle, PANEL_BIAS_Q, &biasQ);
		  GetCtrlVal (panelHandle, PANEL_STATE_I, &stateI);
		  GetCtrlVal (panelHandle, PANEL_STATE_Q, &stateQ);
  		  GetCtrlVal (panelHandle, PANEL_FILT_I, &filtI);
  		  GetCtrlVal (panelHandle, PANEL_FILT_Q, &filtQ);		  
		  GetCtrlVal (panelHandle, PANEL_IMPED, &imped);
  		  GetCtrlVal (panelHandle, PANEL_OUTTYPE, &outType);
		  
		  /* amplitude */
		  if (!outType) {
			  checkERR (rsamiq_SetAmpl (session, 0, amplI));/* I */
			  checkERR (rsamiq_SetAmpl (session, 1, amplQ));/* Q */
		  }
		  else {
			  checkERR (rsamiq_SetAmplBal (session, 0, amplI));/* I */
			  checkERR (rsamiq_SetAmplBal (session, 1, amplQ));/* Q */
		  }
		  
		  /* bias */			  
		  if (outType) {
			  checkERR (rsamiq_SetBias (session, 0, biasI));/* I */
			  checkERR (rsamiq_SetBias (session, 1, biasQ));/* Q */
		  }
		  
		  /* state */
		  checkERR (rsamiq_SetOutState (session, 0, stateI));
		  checkERR (rsamiq_SetOutState (session, 1, stateQ));
		  
		  /* impedance */
		  checkERR (rsamiq_SetImpedance (session, imped));

		  /* filter */
		  checkERR (rsamiq_SetOutFlt (session, 0, filtI));
		  checkERR (rsamiq_SetOutFlt (session, 1, filtQ));

		  /* ouput type */
		  checkERR (rsamiq_SetOutType (session, outType));

			break;
		}
	return 0;
}

/*===========================================================================*/
/* Function: abortApplication
/* Purpose:  
/*===========================================================================*/
long abortApplication (void)
{
    ViChar errorMessage[256], errBuffer[256];
    ViInt16 response;

    SetWaitCursor (0);
    rsamiq_error_message(VI_NULL, checkERR, errorMessage);
    sprintf (errBuffer, "The following error caused the application to fail:\n\n%s\n\n"
    	"Attempting to Close Instrument Driver and Exit Application", errorMessage);
    MessagePopup("ERROR!, Application Failed", errBuffer);
	if ((checkERR = rsamiq_close (session)) < 0) {
        rsamiq_error_message(VI_NULL, checkERR, errorMessage);
        sprintf (errBuffer, "Application failed to close the Instrument Driver.\n\n"
            "%s\n\nExiting Application.", errorMessage);
        MessagePopup("ERROR!",errBuffer);
    }
    else 
        MessagePopup("Application Aborted","Instrument Driver Closed, Exiting Application.");
    QuitUserInterface (checkERR);
    exit (-1);
    
    return 0;
}
