/* Rohde & Schwarz - AMIQ Set Up Trigger ====================================*/
/* Original Release: September 1999											 */
/* Rev 1.0, 09/1999, CVI 5.0.1					 							 */
/* Compatible with instr driver revision: "Rev 1.0, 9/1999, CVI 5.0.1"		 */
/* By: PP, ELCOM, a.s., CZ											 		 */
/*                                                                           */
/* Modification History: 													 */
/*																			 */
/*		None																 */
/*																			 */
/*===========================================================================*/

#include <ansi_c.h>
#include <cvirte.h>		/* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include "Set Up Trigger.h"
#include "export.h"
#include "rsamiq.h"

/*= Definitions =============================================================*/
#define checkERR(fcal) if ((checkERR = fcal) < VI_SUCCESS) {					\
        if (checkERR >= 0xBFFF0000) {							    			\
            abortApplication ();    							    			\
        }														    			\
        else {													    			\
            rsamiq_error_message (VI_NULL, checkERR, errMsg);        			\
            if (checkERR == rsamiq_ERROR_INSTRUMENT_ERROR)		    			\
				rsamiq_error_query (session, &checkERR, errMsg);				\
            SetWaitCursor(0);									    			\
            MessagePopup ("Error", errMsg);						    			\
            return 0;											    			\
        }   													    			\
    } else

/*= Global Variable Definitions =============================================*/
extern ViSession session;
static int panelHandle;
static ViInt32 checkERR;
static ViChar errMsg[256];
static int tsource;
static int tslope;
static int tmode;

#define MODE_OFF	0
#define MODE_GAT	1
#define MODE_SINGLE 2
#define MODE_CONT   3

/*=================== Utility Function Declarations =========================*/
long abortApplication (void);

/*= Main Function ===========================================================*/
int main (int argc, char *argv[])
{
	if (InitCVIRTE (0, argv, 0) == 0)	/* Needed if linking in external compiler; harmless otherwise */
		return -1;	/* out of memory */
	if ((panelHandle = LoadPanel (0, "Set Up Trigger.uir", PANEL)) < 0)
		return -1;
	ConnectionDialog(panelHandle);
	RunUserInterface ();
	return 0;
}

/* Quit user interface */
int CVICALLBACK Quit (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
		  QuitUserInterface(0);	
		  checkERR (rsamiq_close (session));
			break;
	}
	return 0;
}


/* Software trigger */
int CVICALLBACK swTrigg (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			   checkERR (rsamiq_TriggImmediate (session));
			   break;
		}
	return 0;
}

/* Arm AMIQ trigger system */
int CVICALLBACK arm (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			checkERR (rsamiq_TriggArm (session));
			break;
		}
	return 0;
}

/* Abort AMIQ trigger system */
int CVICALLBACK abortTRG (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			checkERR (rsamiq_TriggAbort (session));
			break;
		}
	return 0;
}

/* Set slope mode */
int CVICALLBACK slope (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_VAL_CHANGED:
			GetCtrlVal (panelHandle, PANEL_SLOPE, &tslope);
			checkERR (rsamiq_SetTriggSlope (session, tslope));
			break;
		}
	return 0;
}

/* Set trigger source */
int CVICALLBACK source (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_VAL_CHANGED:
			GetCtrlVal (panelHandle, PANEL_SOURCE, &tsource);
			checkERR (rsamiq_SetTriggSource (session, tsource));
			break;
		}
	return 0;
}

/* Set triger mode */
int CVICALLBACK mode (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_VAL_CHANGED:
			  GetCtrlVal (panelHandle, PANEL_MODE, &tmode);
			  checkERR (rsamiq_SetTriggMode (session, tmode));
			break;
	}
	return 0;
}

/*===========================================================================*/
/* Function: abortApplication
/* Purpose:  
/*===========================================================================*/
long abortApplication (void)
{
    ViChar errorMessage[256], errBuffer[256];
    ViInt16 response;

    SetWaitCursor (0);
    rsamiq_error_message(VI_NULL, checkERR, errorMessage);
    sprintf (errBuffer, "The following error caused the application to fail:\n\n%s\n\n"
    	"Attempting to Close Instrument Driver and Exit Application", errorMessage);
    MessagePopup("ERROR!, Application Failed", errBuffer);
	if ((checkERR = rsamiq_close (session)) < 0) {
        rsamiq_error_message(VI_NULL, checkERR, errorMessage);
        sprintf (errBuffer, "Application failed to close the Instrument Driver.\n\n"
            "%s\n\nExiting Application.", errorMessage);
        MessagePopup("ERROR!",errBuffer);
    }
    else 
        MessagePopup("Application Aborted","Instrument Driver Closed, Exiting Application.");
    QuitUserInterface (checkERR);
    exit (-1);
	
	return 0;
}

/* end of "Set up trigger.c" */

