/* Rohde & Schwarz - AMIQ Transfer File =====================================*/
/* Original Release: September 1999											 */
/* Rev 1.0, 09/1999, CVI 5.0.1					 							 */
/* Compatible with instr driver revision: "Rev 1.0, 9/1999, CVI 5.0.1"		 */
/* By: PP, ELCOM, a.s., CZ											 		 */
/*                                                                           */
/* Modification History: 													 */
/*																			 */
/*		None																 */
/*																			 */
/*===========================================================================*/

#include <utility.h>
#include "rsamiq.h"
#include <ansi_c.h>
#include <cvirte.h>		/* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include "Transfer file.h"
#include "export.h"

/*= Definitions =============================================================*/
#define checkERR(fcal) if ((checkERR = fcal) < VI_SUCCESS) {					\
        if (checkERR >= 0xBFFF0000) {							    			\
            abortApplication ();    							    			\
        }														    			\
        else {													    			\
            rsamiq_error_message (VI_NULL, checkERR, errMsg);        			\
            if (checkERR == rsamiq_ERROR_INSTRUMENT_ERROR)		    			\
				rsamiq_error_query (session, &checkERR, errMsg);				\
            SetWaitCursor(0);									    			\
            MessagePopup ("Error", errMsg);						    			\
            return 0;											    			\
        }   													    			\
    } else

/*= Global Variable Definitions =============================================*/
extern ViSession session; 
static ViInt32 checkERR;
static ViChar errMsg[256];
static int panelHandle;
static int destSw = 0;					
									
static char destPath[MAX_PATHNAME_LEN];
static char path[MAX_PATHNAME_LEN];
static char filename[MAX_FILENAME_LEN];
static char drive[MAX_DRIVENAME_LEN];
static char dir[MAX_DIRNAME_LEN];

/*=================== Utility Function Declarations =========================*/
long abortApplication (void);

/*= Main Function ===========================================================*/
int main (int argc, char *argv[])
{
	if (InitCVIRTE (0, argv, 0) == 0)	/* Needed if linking in external compiler; harmless otherwise */
		return -1;	/* out of memory */
	if ((panelHandle = LoadPanel (0, "Transfer file.uir", PANEL)) < 0)
		return -1;
	ConnectionDialog(panelHandle);
	RunUserInterface ();
	return 0;
}

/* Invoke file dialog */
int CVICALLBACK InvokeFileDialog (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_LEFT_CLICK:
 		  if(FileSelectPopup ("", "*.wv", "", "Waveform file", VAL_LOAD_BUTTON, 0,
							  1, 1, 0, path))
		  {
			SetCtrlVal (panelHandle, PANEL_FILENAME, path);
			SplitPath (path, drive, dir, filename);
			SetCtrlVal (panelHandle, PANEL_AMIQF, filename);
			SetCtrlAttribute (panelHandle, PANEL_TRANS, ATTR_DIMMED, 0);
			
		  }
			break;
	}
	return 0;
}

/* Quit user interface */
int CVICALLBACK Quit (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			rsamiq_close (session);
			QuitUserInterface (0);
			break;
	}
	return 0;
}

/* Select destination */
int CVICALLBACK dest (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_VAL_CHANGED:
		  GetCtrlVal (panelHandle, PANEL_DEST, &destSw);
		  SetCtrlAttribute (panelHandle, PANEL_AMIQF, ATTR_DIMMED, !destSw);  			  
			break;
		}
	return 0;
}

/* Transfer file to selected destination */
int CVICALLBACK trans (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			SetWaitCursor (1);
			if(destSw){
			  	GetCtrlVal (panelHandle, PANEL_AMIQF, destPath);
			  	if(rsamiq_WaveformFileToDisk (session, path, destPath))
				  MessagePopup ("Transmission Error !",
				  				"Unable to transfer data");
			  } 
			  else
			  {
			  	if(rsamiq_WaveformFileToRAM (session, path)) 
			      MessagePopup ("Transmission Error !", 
			        			"Unable to transfer data");

	  			checkERR( rsamiq_SetOutState (session, 0, 1));
				checkERR( rsamiq_SetOutState (session, 1, 1));
				checkERR( rsamiq_TriggArm (session));
				checkERR( rsamiq_TriggImmediate (session));
			  }
			SetWaitCursor (0);
			break;
		}
	return 0;
}

/*===========================================================================*/
/* Function: abortApplication
/* Purpose:  
/*===========================================================================*/
long abortApplication (void)
{
    ViChar errorMessage[256], errBuffer[256];
    ViInt16 response;

    SetWaitCursor (0);
    rsamiq_error_message(VI_NULL, checkERR, errorMessage);
    sprintf (errBuffer, "The following error caused the application to fail:\n\n%s\n\n"
    	"Attempting to Close Instrument Driver and Exit Application", errorMessage);
    MessagePopup("ERROR!, Application Failed", errBuffer);
	if ((checkERR = rsamiq_close (session)) < 0) {
        rsamiq_error_message(VI_NULL, checkERR, errorMessage);
        sprintf (errBuffer, "Application failed to close the Instrument Driver.\n\n"
            "%s\n\nExiting Application.", errorMessage);
        MessagePopup("ERROR!",errBuffer);
    }
    else 
        MessagePopup("Application Aborted","Instrument Driver Closed, Exiting Application.");
    QuitUserInterface (checkERR);
    exit (-1);
    
    return 0;
}

/* End of "Transfer file.c" */
